<?php

namespace ALFA\ALFAPayment\Controller\Url;

use Magento\Framework\App\Action\Action;
use Magento\Framework\App\Action\Context;
use Magento\Sales\Model\Order;
use Magento\Framework\App\CsrfAwareActionInterface;
use Magento\Framework\App\Request\InvalidRequestException;
use Magento\Framework\App\RequestInterface;
use Magento\Framework\Controller\Result\JsonFactory;
use Magento\Framework\DataObjectFactory;
use Magento\Sales\Model\Order\Payment;
use Magento\Sales\Model\OrderRepository as MageOrderRepository;


class Execute extends Action implements CsrfAwareActionInterface
{
    /** @var \Magento\Framework\View\Result\PageFactory */
    protected $resultPageFactory;

    protected $_encryptor;
    protected $_urlBuilder;
    protected $_customLogger;
    protected $orderRepository;
    protected $module_version_str = "1.4.0";

    protected $alfa_form;


    public function __construct(
        Context $context,
        \Magento\Framework\View\Result\PageFactory $resultPageFactory,
        \Magento\Framework\Encryption\EncryptorInterface $encryptor,
        \Magento\Framework\App\ResponseFactory $responseFactory,
        \Magento\Framework\UrlInterface $urlBuilder,
        \Magento\Sales\Api\OrderRepositoryInterface $orderRepository,
        \ALFA\ALFAPayment\Block\Widget\Redirect $alfa_form,
        \ALFA\ALFAPayment\Logger\Logger $customLogger
    )
    {
        parent::__construct($context);
        $this->_encryptor = $encryptor;
        $this->resultPageFactory = $resultPageFactory;
        $this->_responseFactory = $responseFactory;
        $this->_urlBuilder = $urlBuilder;
        $this->orderRepository = $orderRepository;

        $this->_customLogger = $customLogger;
        $this->alfa_form = $alfa_form;
    }


    /**
     * Load the page defined
     *
     * @return \Magento\Framework\View\Result\Page
     */
    public function execute()
    {

        //load model
        /* @var $paymentMethod \Magento\Authorizenet\Model\DirectPost */
        $paymentMethod = $this->_objectManager->create('ALFA\ALFAPayment\Model\ALFAPayment');

//        $post_data = $this->getRequest()->getPostValue();
        $post_data = $this->alfa_form->getPostData();

        // Detect the version of this CMS
        $objectManager = \Magento\Framework\App\ObjectManager::getInstance();
        $productMetadata = $objectManager->get('Magento\Framework\App\ProductMetadataInterface');
        $version = $productMetadata->getVersion();

        $args['orderNumber'] = $post_data['order_id'] . "#" . time();
        $args['description'] = __("Pay order №") . $post_data['order_id'];
        $args['userName'] = $paymentMethod->getConfigData("ALFA_MERCHANT_LOGIN");
        $args['password'] = $this->_encryptor->decrypt($paymentMethod->getConfigData("ALFA_MERCHANT_PASSWORD"));
        $args['returnUrl'] = $this->_urlBuilder->getUrl('alfapayment/url/success');

        //todo reset this if send_order is enabled
        $args['amount'] = $post_data['amount']; //already comes from other place =) * 100;

        $jsonParams_array = array(
            'CMS:' => 'Magento ' . $version,
            'Module-Version: ' => 'ALFAPayment ' . $this->module_version_str,
        );

        // IF send_order
        if ($paymentMethod->getConfigData('send_order')) {

            $args['taxSystem'] = $paymentMethod->getConfigData('tax_system');

            $alfa_items = array();

            $order = $this->_objectManager->create('Magento\Sales\Model\Order')->loadByIncrementId($post_data['order_id']);

            $cn = 1;
            foreach ($order->getAllVisibleItems() as $item) {

                $item_data = $item->getData();

                $alfa_item_price = ceil($item_data['price'] * 100);

                $item_rate = (int)$item_data['tax_percent'];
                $alfa_item_tax_type = 0;
                switch ($item_rate) {
                    case 20:
                        $alfa_item_tax_type = 6;
                        break;
                    case 18:
                        $alfa_item_tax_type = 3;
                        break;
                    case 10:
                        $alfa_item_tax_type = 2;
                        break;
                    case 0:
                        $alfa_item_tax_type = 1;
                        break;
                }

                $alfa_item['positionId'] = $cn;
                $alfa_item['name'] = $item_data['name'];
                $alfa_item['quantity'] = array(
                    'value' => round($item_data['qty_ordered'], 2),
                    'measure' => 'piece'
                );
                $alfa_item['itemAmount'] = $alfa_item_price * $item_data['qty_ordered'];
                $alfa_item['itemCode'] = $item_data['product_id'];
                $alfa_item['tax'] = array(
                    'taxType' => $alfa_item_tax_type
                );
                $alfa_item['itemPrice'] = $alfa_item_price;
                $cn++;

                // FFD 1.05 added
//                if ($paymentMethod->getConfigData('ffd_version') == 'v105') {

                    $attributes = array();
                    $attributes[] = array(
                        "name" => "paymentMethod",
                        "value" => $paymentMethod->getConfigData('ffd_paymentMethodType')
                    );
                    $attributes[] = array(
                        "name" => "paymentObject",
                        "value" => $paymentMethod->getConfigData('ffd_paymentObjectType')
                    );
                    $alfa_item['itemAttributes']['attributes'] = $attributes;
//                }
                $alfa_items[] = $alfa_item;
            }

            // DISCOUNT
            //todo


            // DELIVERY
            if ($order->getShippingAmount() > 0) {

                $itemShipment['positionId'] = $cn;
                $itemShipment['name'] = __("Delivery");
                $itemShipment['quantity'] = array(
                    'value' => 1,
                    'measure' => 'piece'
                );
                $itemShipment['itemAmount'] = $itemShipment['itemPrice'] = ceil($order->getShippingAmount() * 100);
                $itemShipment['itemCode'] = __("Delivery");

                //todo
                $itemShipment['tax'] = array(
                    'taxType' => 0
                );

                // FFD 1.05 added
//                if ($paymentMethod->getConfigData('ffd_version') == 'v105') {

                    $attributes = array();
                    $attributes[] = array(
                        "name" => "paymentMethod",
                        "value" => $paymentMethod->getConfigData('ffd_paymentMethodType')
                    );
                    $attributes[] = array(
                        "name" => "paymentObject",
                        "value" => 4
                    );

                    $itemShipment['itemAttributes']['attributes'] = $attributes;
//                }
                $alfa_items[] = $itemShipment;
            }

            /* Создание и заполнение массива данных заказа для фискализации */
            $order_bundle = array(
                'orderCreationDate' => time(), //$order->getCreatedAt(), //todo maybe =)
                'customerDetails' => array(
                    'email' => $order->getCustomerEmail(),
                    'phone' => preg_replace('/\D+/', '', $order->getShippingAddress()->getTelephone())
                ),
                'cartItems' => array('items' => $alfa_items)
            );


            /* Заполнение массива данных для запроса c фискализацией */
            $args['orderBundle'] = json_encode($order_bundle);


        } // send_order IF


        $args['jsonParams'] = json_encode($jsonParams_array);

        // EIRAno, How do you know that? =)
        $ch = curl_init();
        curl_setopt_array($ch, array(
                CURLOPT_HTTPHEADER => array(
                    'CMS: Magento ' . $version,
                    'Module-Version: ' . $this->module_version_str
                ),
                CURLOPT_URL => $paymentMethod->getGateUrl(),
                CURLOPT_RETURNTRANSFER => true,
                CURLOPT_SSL_VERIFYPEER => false,
                CURLOPT_POST => true,
                CURLOPT_POSTFIELDS => $args, '', '&')
        );

        $response = curl_exec($ch);
        curl_close($ch);
//die(json_encode($response));
        $this->_customLogger->info("[REQUEST] " . json_encode($args));
        $this->_customLogger->info("[RESPONSE] " . $response);
        $response = json_decode($response, true);

        $CustomRedirectionUrl = $this->_urlBuilder->getUrl($response['formUrl']);
        $this->_responseFactory->create()->setRedirect($CustomRedirectionUrl)->sendResponse();

        exit();

    }
    /**
     * Create exception in case CSRF validation failed.
     * Return null if default exception will suffice.
     *
     * @param RequestInterface $request
     *
     * @return InvalidRequestException|null
     * @SuppressWarnings(PMD.UnusedFormalParameter)
     */
    public function createCsrfValidationException(RequestInterface $request): ?InvalidRequestException
    {
        return null;
    }

    /**
     * Perform custom request validation.
     * Return null if default validation is needed.
     *
     * @param RequestInterface $request
     *
     * @return bool|null
     * @SuppressWarnings(PMD.UnusedFormalParameter)
     */
    public function validateForCsrf(RequestInterface $request): ?bool
    {
        return true;
    }
}
